@@ -3,6 +3,7 @@ package cmd
 import (
 	"context"
 	"fmt"
+	"log"
 	"strings"
 	"sync"
 	"time"
@@ -15,6 +16,7 @@ import (
 	"github.com/turbot/steampipe/pkg/constants"
 	"github.com/turbot/steampipe/pkg/db/db_local"
 	"github.com/turbot/steampipe/pkg/display"
+	"github.com/turbot/steampipe/pkg/error_helpers"
 	"github.com/turbot/steampipe/pkg/ociinstaller"
 	"github.com/turbot/steampipe/pkg/ociinstaller/versionfile"
 	"github.com/turbot/steampipe/pkg/plugin"
@@ -190,7 +192,7 @@ func runPluginInstallCmd(cmd *cobra.Command, args []string) {
 	defer func() {
 		utils.LogTime("runPluginInstallCmd end")
 		if r := recover(); r != nil {
-			utils.ShowError(ctx, helpers.ToError(r))
+			error_helpers.ShowError(ctx, helpers.ToError(r))
 			exitCode = constants.ExitCodeUnknownErrorPanic
 		}
 	}()
@@ -203,7 +205,7 @@ func runPluginInstallCmd(cmd *cobra.Command, args []string) {
 
 	if len(plugins) == 0 {
 		fmt.Println()
-		utils.ShowError(ctx, fmt.Errorf("you need to provide at least one plugin to install"))
+		error_helpers.ShowError(ctx, fmt.Errorf("you need to provide at least one plugin to install"))
 		fmt.Println()
 		cmd.Help()
 		fmt.Println()
@@ -286,7 +288,7 @@ func runPluginUpdateCmd(cmd *cobra.Command, args []string) {
 	defer func() {
 		utils.LogTime("runPluginUpdateCmd end")
 		if r := recover(); r != nil {
-			utils.ShowError(ctx, helpers.ToError(r))
+			error_helpers.ShowError(ctx, helpers.ToError(r))
 			exitCode = constants.ExitCodeUnknownErrorPanic
 		}
 	}()
@@ -297,7 +299,7 @@ func runPluginUpdateCmd(cmd *cobra.Command, args []string) {
 	plugins, err := resolveUpdatePluginsFromArgs(args)
 	if err != nil {
 		fmt.Println()
-		utils.ShowError(ctx, err)
+		error_helpers.ShowError(ctx, err)
 		fmt.Println()
 		cmd.Help()
 		fmt.Println()
@@ -308,14 +310,14 @@ func runPluginUpdateCmd(cmd *cobra.Command, args []string) {
 	if len(plugins) > 0 && !(cmdconfig.Viper().GetBool("all")) && plugins[0] == "all" {
 		// improve the response to wrong argument "steampipe plugin update all"
 		fmt.Println()
-		utils.ShowError(ctx, fmt.Errorf("Did you mean %s?", constants.Bold("--all")))
+		error_helpers.ShowError(ctx, fmt.Errorf("Did you mean %s?", constants.Bold("--all")))
 		fmt.Println()
 		return
 	}
 
 	state, err := statefile.LoadState()
 	if err != nil {
-		utils.ShowError(ctx, fmt.Errorf("could not load state"))
+		error_helpers.ShowError(ctx, fmt.Errorf("could not load state"))
 		exitCode = constants.ExitCodeLoadingError
 		return
 	}
@@ -323,7 +325,7 @@ func runPluginUpdateCmd(cmd *cobra.Command, args []string) {
 	// load up the version file data
 	versionData, err := versionfile.LoadPluginVersionFile()
 	if err != nil {
-		utils.ShowError(ctx, fmt.Errorf("error loading current plugin data"))
+		error_helpers.ShowError(ctx, fmt.Errorf("error loading current plugin data"))
 		exitCode = constants.ExitCodeLoadingError
 		return
 	}
@@ -378,7 +380,7 @@ func runPluginUpdateCmd(cmd *cobra.Command, args []string) {
 	if len(reports) == 0 {
 		// this happens if for some reason the update server could not be contacted,
 		// in which case we get back an empty map
-		utils.ShowError(ctx, fmt.Errorf("there was an issue contacting the update server. Please try later."))
+		error_helpers.ShowError(ctx, fmt.Errorf("there was an issue contacting the update server, please try later"))
 		exitCode = constants.ExitCodeLoadingError
 		return
 	}
@@ -388,7 +390,7 @@ func runPluginUpdateCmd(cmd *cobra.Command, args []string) {
 	progressBars := uiprogress.New()
 	progressBars.Start()
 
-	sorted := utils.SortedStringKeys(reports)
+	sorted := utils.SortedMapKeys(reports)
 	for _, key := range sorted {
 		report := reports[key]
 		updateWaitGroup.Add(1)
@@ -538,14 +540,14 @@ func refreshConnectionsIfNecessary(ctx context.Context, reports display.PluginIn
 	// reload the config, since an installation MUST have created a new config file
 	if shouldReload {
 		var cmd = viper.Get(constants.ConfigKeyActiveCommand).(*cobra.Command)
-		config, err := steampipeconfig.LoadSteampipeConfig(viper.GetString(constants.ArgWorkspaceChDir), cmd.Name())
+		config, err := steampipeconfig.LoadSteampipeConfig(viper.GetString(constants.ArgModLocation), cmd.Name())
 		if err != nil {
 			return err
 		}
 		steampipeconfig.GlobalConfig = config
 	}
 
-	client, err := db_local.GetLocalClient(ctx, constants.InvokerPlugin)
+	client, err := db_local.GetLocalClient(ctx, constants.InvokerPlugin, nil)
 	if err != nil {
 		return err
 	}
@@ -565,29 +567,69 @@ func runPluginListCmd(cmd *cobra.Command, args []string) {
 	defer func() {
 		utils.LogTime("runPluginListCmd end")
 		if r := recover(); r != nil {
-			utils.ShowError(ctx, helpers.ToError(r))
+			error_helpers.ShowError(ctx, helpers.ToError(r))
 			exitCode = constants.ExitCodeUnknownErrorPanic
 		}
 	}()
 
-	pluginConnectionMap, err := getPluginConnectionMap(cmd.Context())
+	pluginConnectionMap, res, err := getPluginConnectionMap(ctx)
 	if err != nil {
-		utils.ShowErrorWithMessage(ctx, err, "Plugin Listing failed")
+		error_helpers.ShowErrorWithMessage(ctx, err, "plugin listing failed")
 		exitCode = constants.ExitCodePluginListFailure
 		return
 	}
 
+	missingPluginMap := res.Updates.MissingPlugins
+	log.Printf("[TRACE] missing plugins: %v", missingPluginMap)
+
 	list, err := plugin.List(pluginConnectionMap)
 	if err != nil {
-		utils.ShowErrorWithMessage(ctx, err, "Plugin Listing failed")
+		error_helpers.ShowErrorWithMessage(ctx, err, "plugin listing failed")
 		exitCode = constants.ExitCodePluginListFailure
+		return
 	}
-	headers := []string{"Name", "Version", "Connections"}
-	rows := [][]string{}
-	for _, item := range list {
-		rows = append(rows, []string{item.Name, item.Version, strings.Join(item.Connections, ",")})
+
+	// If there are missing plugins which have connections left over, list them
+	// along with installed plugins
+	if len(missingPluginMap) != 0 {
+		// List installed plugins
+		if len(list) != 0 {
+			headers := []string{"Installed Plugin", "Version", "Connections"}
+			rows := [][]string{}
+			for _, item := range list {
+				rows = append(rows, []string{item.Name, item.Version, strings.Join(item.Connections, ",")})
+			}
+			display.ShowWrappedTable(headers, rows, &display.ShowWrappedTableOptions{AutoMerge: false})
+			fmt.Printf("\n")
+		}
+
+		// List missing plugins
+		headers := []string{"Missing Plugin", "Connections"}
+		var conns = []string{}
+		var missingRows [][]string
+		for p, item := range missingPluginMap {
+			for _, conn := range item {
+				conns = append(conns, conn.Name)
+			}
+			missingRows = append(missingRows, []string{p, strings.Join(conns, ",")})
+			conns = []string{}
+		}
+		display.ShowWrappedTable(headers, missingRows, &display.ShowWrappedTableOptions{AutoMerge: false})
+	} else {
+		headers := []string{"Installed Plugin", "Version", "Connections"}
+		rows := [][]string{}
+		for _, item := range list {
+			rows = append(rows, []string{item.Name, item.Version, strings.Join(item.Connections, ",")})
+		}
+		display.ShowWrappedTable(headers, rows, &display.ShowWrappedTableOptions{AutoMerge: false})
+	}
+
+	// display any initialisation warnings
+	if len(res.Warnings) > 0 {
+		fmt.Println()
+		res.ShowWarnings()
+		fmt.Printf("\n")
 	}
-	display.ShowWrappedTable(headers, rows, false)
 }
 
 func runPluginUninstallCmd(cmd *cobra.Command, args []string) {
@@ -597,14 +639,14 @@ func runPluginUninstallCmd(cmd *cobra.Command, args []string) {
 	defer func() {
 		utils.LogTime("runPluginUninstallCmd end")
 		if r := recover(); r != nil {
-			utils.ShowError(ctx, helpers.ToError(r))
+			error_helpers.ShowError(ctx, helpers.ToError(r))
 			exitCode = constants.ExitCodeUnknownErrorPanic
 		}
 	}()
 
 	if len(args) == 0 {
 		fmt.Println()
-		utils.ShowError(ctx, fmt.Errorf("you need to provide at least one plugin to uninstall"))
+		error_helpers.ShowError(ctx, fmt.Errorf("you need to provide at least one plugin to uninstall"))
 		fmt.Println()
 		cmd.Help()
 		fmt.Println()
@@ -612,9 +654,9 @@ func runPluginUninstallCmd(cmd *cobra.Command, args []string) {
 		return
 	}
 
-	connectionMap, err := getPluginConnectionMap(ctx)
+	connectionMap, _, err := getPluginConnectionMap(ctx)
 	if err != nil {
-		utils.ShowError(ctx, err)
+		error_helpers.ShowError(ctx, err)
 		exitCode = constants.ExitCodePluginListFailure
 		return
 	}
@@ -624,7 +666,7 @@ func runPluginUninstallCmd(cmd *cobra.Command, args []string) {
 	for _, p := range args {
 		spinner.SetStatus(fmt.Sprintf("Uninstalling %s", p))
 		if report, err := plugin.Remove(ctx, p, connectionMap); err != nil {
-			utils.ShowErrorWithMessage(ctx, err, fmt.Sprintf("Failed to uninstall plugin '%s'", p))
+			error_helpers.ShowErrorWithMessage(ctx, err, fmt.Sprintf("Failed to uninstall plugin '%s'", p))
 		} else {
 			report.ShortName = p
 			reports = append(reports, *report)
@@ -634,22 +676,18 @@ func runPluginUninstallCmd(cmd *cobra.Command, args []string) {
 	reports.Print()
 }
 
-// returns a map of pluginFullName -> []{connections using pluginFullName}
-func getPluginConnectionMap(ctx context.Context) (map[string][]modconfig.Connection, error) {
-	client, err := db_local.GetLocalClient(ctx, constants.InvokerPlugin)
+func getPluginConnectionMap(ctx context.Context) (map[string][]modconfig.Connection, *steampipeconfig.RefreshConnectionResult, error) {
+	client, err := db_local.GetLocalClient(ctx, constants.InvokerPlugin, nil)
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 	defer client.Close(ctx)
 	res := client.RefreshConnectionAndSearchPaths(ctx)
 	if res.Error != nil {
-		return nil, res.Error
+		return nil, nil, res.Error
 	}
-	// display any initialisation warnings
-	res.ShowWarnings()
 
 	pluginConnectionMap := make(map[string][]modconfig.Connection)
-
 	for _, v := range *client.ConnectionMap() {
 		_, found := pluginConnectionMap[v.Plugin]
 		if !found {
@@ -657,5 +695,6 @@ func getPluginConnectionMap(ctx context.Context) (map[string][]modconfig.Connect
 		}
 		pluginConnectionMap[v.Plugin] = append(pluginConnectionMap[v.Plugin], *v.Connection)
 	}
-	return pluginConnectionMap, nil
+
+	return pluginConnectionMap, res, nil
 }
