@@ -17,14 +17,15 @@ type DashboardNodeRun interface {
 	Initialise(ctx context.Context)
 	Execute(ctx context.Context)
 	GetName() string
+	GetTitle() string
 	GetRunStatus() DashboardRunStatus
-	SetError(err error)
+	SetError(context.Context, error)
 	GetError() error
-	SetComplete()
+	SetComplete(context.Context)
 	RunComplete() bool
 	GetChildren() []DashboardNodeRun
 	ChildrenComplete() bool
-	GetInputsDependingOn(changedInputName string) []string
+	GetInputsDependingOn(string) []string
 	AsTreeNode() *SnapshotTreeNode
 }
 
