@@ -10,6 +10,7 @@ import (
 	"github.com/turbot/go-kit/helpers"
 	"github.com/turbot/steampipe/pkg/constants"
 	"github.com/turbot/steampipe/pkg/db/db_common"
+	"github.com/turbot/steampipe/pkg/error_helpers"
 	"github.com/turbot/steampipe/pkg/utils"
 	"github.com/turbot/steampipe/pkg/workspace"
 )
@@ -25,7 +26,7 @@ func (c *InteractiveClient) handleInitResult(ctx context.Context, initResult *db
 		c.ClosePrompt(AfterPromptCloseExit)
 		// add newline to ensure error is not printed at end of current prompt line
 		fmt.Println()
-		utils.ShowError(ctx, initResult.Error)
+		error_helpers.ShowError(ctx, initResult.Error)
 		return
 	}
 
@@ -33,14 +34,14 @@ func (c *InteractiveClient) handleInitResult(ctx context.Context, initResult *db
 		c.ClosePrompt(AfterPromptCloseExit)
 		// add newline to ensure error is not printed at end of current prompt line
 		fmt.Println()
-		utils.ShowError(ctx, initResult.Error)
+		error_helpers.ShowError(ctx, initResult.Error)
 		log.Printf("[TRACE] prompt context has been cancelled - not handling init result")
 		return
 	}
 
 	if initResult.HasMessages() {
-		c.hidePrompt = true
 		c.interactivePrompt.ClearLine()
+		c.hidePrompt = true
 		fmt.Println()
 		initResult.DisplayMessages()
 		c.hidePrompt = false
@@ -49,15 +50,20 @@ func (c *InteractiveClient) handleInitResult(ctx context.Context, initResult *db
 		c.interactivePrompt.Render()
 	}
 
+	c.initialiseSuggestions()
 	// tell the workspace to reset the prompt after displaying async filewatcher messages
-	c.initData.Workspace.SetOnFileWatcherEventMessages(func() { c.interactivePrompt.Render() })
+	c.initData.Workspace.SetOnFileWatcherEventMessages(func() {
+		c.initialiseSuggestions()
+		c.interactivePrompt.Render()
+	})
+
 }
 
 func (c *InteractiveClient) readInitDataStream(ctx context.Context) {
 	defer func() {
 		if r := recover(); r != nil {
 			c.interactivePrompt.ClearScreen()
-			utils.ShowError(ctx, helpers.ToError(r))
+			error_helpers.ShowError(ctx, helpers.ToError(r))
 		}
 	}()
 
@@ -85,11 +91,15 @@ func (c *InteractiveClient) readInitDataStream(ctx context.Context) {
 		}
 	}
 
+	// Trigger a re-render of the prompt, so that the prompt actually shows up,
+	// since the prompt may have been removed by the installation spinner
+	c.interactivePrompt.Render()
+
 }
 
 func (c *InteractiveClient) workspaceWatcherErrorHandler(ctx context.Context, err error) {
 	fmt.Println()
-	utils.ShowError(ctx, err)
+	error_helpers.ShowError(ctx, err)
 	c.interactivePrompt.Render()
 }
 
