@@ -49,13 +49,10 @@ type Dashboard struct {
 	// TODO [reports] can a dashboard ever have multiple parents??
 	parents                []ModTreeItem
 	runtimeDependencyGraph *topsort.Graph
-
-	HclType string
 }
 
-func NewDashboard(block *hcl.Block, mod *Mod, shortName string) *Dashboard {
+func NewDashboard(block *hcl.Block, mod *Mod, shortName string) HclResource {
 	c := &Dashboard{
-		HclType:         block.Type,
 		ShortName:       shortName,
 		FullName:        fmt.Sprintf("%s.%s.%s", mod.ShortName, block.Type, shortName),
 		UnqualifiedName: fmt.Sprintf("%s.%s", block.Type, shortName),
@@ -68,6 +65,40 @@ func NewDashboard(block *hcl.Block, mod *Mod, shortName string) *Dashboard {
 	return c
 }
 
+// NewQueryDashboard creates a dashboard to wrap a query/control
+// this is used for snapshot generation
+func NewQueryDashboard(q ModTreeItem) (*Dashboard, error) {
+	parsedName, err := ParseResourceName(q.Name())
+	if err != nil {
+		return nil, err
+	}
+	dashboardName := BuildFullResourceName(q.GetMod().ShortName, BlockTypeDashboard, parsedName.Name)
+
+	var dashboard = &Dashboard{
+		ResourceWithMetadataBase: ResourceWithMetadataBase{
+			metadata: &ResourceMetadata{},
+		},
+		ShortName:       parsedName.Name,
+		FullName:        dashboardName,
+		UnqualifiedName: fmt.Sprintf("%s.%s", BlockTypeDashboard, parsedName),
+		Title:           utils.ToStringPointer(q.GetTitle()),
+		Description:     utils.ToStringPointer(q.GetDescription()),
+		Documentation:   utils.ToStringPointer(q.GetDocumentation()),
+		Tags:            q.GetTags(),
+		Mod:             q.GetMod(),
+	}
+
+	dashboard.setUrlPath()
+
+	chart, err := NewQueryDashboardTable(q)
+	if err != nil {
+		return nil, err
+	}
+	dashboard.children = []ModTreeItem{chart}
+
+	return dashboard, nil
+}
+
 func (d *Dashboard) setUrlPath() {
 	d.UrlPath = fmt.Sprintf("/%s", d.FullName)
 }
@@ -88,7 +119,7 @@ func (d *Dashboard) Name() string {
 }
 
 // OnDecoded implements HclResource
-func (d *Dashboard) OnDecoded(block *hcl.Block, resourceMapProvider ModResourcesProvider) hcl.Diagnostics {
+func (d *Dashboard) OnDecoded(block *hcl.Block, resourceMapProvider ResourceMapsProvider) hcl.Diagnostics {
 	d.setBaseProperties(resourceMapProvider)
 
 	d.ChildNames = make([]string, len(d.children))
@@ -99,17 +130,17 @@ func (d *Dashboard) OnDecoded(block *hcl.Block, resourceMapProvider ModResources
 	return nil
 }
 
-// AddReference implements HclResource
+// AddReference implements ResourceWithMetadata
 func (d *Dashboard) AddReference(ref *ResourceReference) {
 	d.References = append(d.References, ref)
 }
 
-// GetReferences implements HclResource
+// GetReferences implements ResourceWithMetadata
 func (d *Dashboard) GetReferences() []*ResourceReference {
 	return d.References
 }
 
-// GetMod implements HclResource
+// GetMod implements ModTreeItem
 func (d *Dashboard) GetMod() *Mod {
 	return d.Mod
 }
@@ -119,6 +150,11 @@ func (d *Dashboard) GetDeclRange() *hcl.Range {
 	return &d.DeclRange
 }
 
+// BlockType implements HclResource
+func (*Dashboard) BlockType() string {
+	return BlockTypeDashboard
+}
+
 // AddParent implements ModTreeItem
 func (d *Dashboard) AddParent(parent ModTreeItem) error {
 	d.parents = append(d.parents, parent)
@@ -135,7 +171,7 @@ func (d *Dashboard) GetChildren() []ModTreeItem {
 	return d.children
 }
 
-// GetTitle implements ModTreeItem
+// GetTitle implements HclResource
 func (d *Dashboard) GetTitle() string {
 	return typehelpers.SafeString(d.Title)
 }
@@ -145,9 +181,12 @@ func (d *Dashboard) GetDescription() string {
 	return ""
 }
 
-// GetTags implements ModTreeItem
+// GetTags implements HclResource
 func (d *Dashboard) GetTags() map[string]string {
-	return nil
+	if d.Tags != nil {
+		return d.Tags
+	}
+	return map[string]string{}
 }
 
 // GetPaths implements ModTreeItem
@@ -168,6 +207,11 @@ func (d *Dashboard) SetPaths() {
 	}
 }
 
+// GetDocumentation implement ModTreeItem
+func (d *Dashboard) GetDocumentation() string {
+	return typehelpers.SafeString(d.Documentation)
+}
+
 func (d *Dashboard) Diff(other *Dashboard) *DashboardTreeItemDiffs {
 	res := &DashboardTreeItemDiffs{
 		Item: d,
@@ -240,7 +284,7 @@ func (d *Dashboard) WalkResources(resourceFunc func(resource HclResource) (bool,
 	return nil
 }
 
-func (d *Dashboard) BuildRuntimeDependencyTree(workspace ModResourcesProvider) error {
+func (d *Dashboard) BuildRuntimeDependencyTree(workspace ResourceMapsProvider) error {
 	d.runtimeDependencyGraph = topsort.NewGraph()
 	// add root node - this will depend on all other nodes
 	d.runtimeDependencyGraph.AddNode(rootRuntimeDependencyNode)
@@ -375,7 +419,7 @@ func (d *Dashboard) setInputMap() []string {
 	return duplicates
 }
 
-func (d *Dashboard) setBaseProperties(resourceMapProvider ModResourcesProvider) {
+func (d *Dashboard) setBaseProperties(resourceMapProvider ResourceMapsProvider) {
 	// not all base properties are stored in the evalContext
 	// (e.g. resource metadata and runtime dependencies are not stores)
 	//  so resolve base from the resource map provider (which is the RunContext)
@@ -400,7 +444,7 @@ func (d *Dashboard) setBaseProperties(resourceMapProvider ModResourcesProvider)
 
 	d.addBaseInputs(d.Base.Inputs)
 
-	d.Tags = utils.MergeStringMaps(d.Tags, d.Base.Tags)
+	d.Tags = utils.MergeMaps(d.Tags, d.Base.Tags)
 
 	if d.Description == nil {
 		d.Description = d.Base.Description
