@@ -13,9 +13,11 @@ import (
 	"github.com/fsnotify/fsnotify"
 	"github.com/spf13/viper"
 	filehelpers "github.com/turbot/go-kit/files"
+	"github.com/turbot/go-kit/filewatcher"
 	"github.com/turbot/steampipe/pkg/constants"
 	"github.com/turbot/steampipe/pkg/dashboard/dashboardevents"
 	"github.com/turbot/steampipe/pkg/db/db_common"
+	"github.com/turbot/steampipe/pkg/error_helpers"
 	"github.com/turbot/steampipe/pkg/filepaths"
 	"github.com/turbot/steampipe/pkg/modinstaller"
 	"github.com/turbot/steampipe/pkg/steampipeconfig"
@@ -33,25 +35,33 @@ type Workspace struct {
 	Mods map[string]*modconfig.Mod
 	// the input variables used in the parse
 	VariableValues map[string]string
+	CloudMetadata  *steampipeconfig.CloudMetadata
 
-	CloudMetadata *steampipeconfig.CloudMetadata
-	watcher       *utils.FileWatcher
-	loadLock      sync.Mutex
-	exclusions    []string
-	modFilePath   string
+	// source snapshot paths
+	// if this is set, no other mod resources are loaded and
+	// the ResourceMaps returned by GetModResources will contain only the snapshots
+	SourceSnapshots []string
+
+	watcher     *filewatcher.FileWatcher
+	loadLock    sync.Mutex
+	exclusions  []string
+	modFilePath string
 	// should we load/watch files recursively
 	listFlag                filehelpers.ListFlag
 	fileWatcherErrorHandler func(context.Context, error)
 	watcherError            error
 	// event handlers
 	dashboardEventHandlers []dashboardevents.DashboardEventHandler
-	// callback function to reset display after the file watche displays messages
+	// callback function called when there is a file watcher event
 	onFileWatcherEventMessages func()
 	loadPseudoResources        bool
 	// channel used to send dashboard events to the handleDashbooardEvent goroutine
 	dashboardEventChan chan dashboardevents.DashboardEvent
 	// count of workspace changed events - used to ignore first event
 	changeEventCount int
+	// avoid concurrent map access when multiple db connections may try to access preparedStatementFailures
+	preparedStatementFailureLock sync.Mutex
+	preparedStatementFailures    map[string]*steampipeconfig.PreparedStatementFailure
 }
 
 // Load creates a Workspace and loads the workspace mod
@@ -137,7 +147,7 @@ func LoadResourceNames(workspacePath string) (*modconfig.WorkspaceResources, err
 }
 
 func (w *Workspace) SetupWatcher(ctx context.Context, client db_common.Client, errorHandler func(context.Context, error)) error {
-	watcherOptions := &utils.WatcherOptions{
+	watcherOptions := &filewatcher.WatcherOptions{
 		Directories: []string{w.Path},
 		Include:     filehelpers.InclusionsFromExtensions(steampipeconfig.GetModFileExtensions()),
 		Exclude:     w.exclusions,
@@ -150,7 +160,7 @@ func (w *Workspace) SetupWatcher(ctx context.Context, client db_common.Client, e
 			w.handleFileWatcherEvent(ctx, client, events)
 		},
 	}
-	watcher, err := utils.NewWatcher(watcherOptions)
+	watcher, err := filewatcher.NewWatcher(watcherOptions)
 	if err != nil {
 		return err
 	}
@@ -164,7 +174,7 @@ func (w *Workspace) SetupWatcher(ctx context.Context, client db_common.Client, e
 	if w.fileWatcherErrorHandler == nil {
 		w.fileWatcherErrorHandler = func(ctx context.Context, err error) {
 			fmt.Println()
-			utils.ShowErrorWithMessage(ctx, err, "Failed to reload mod from file watcher")
+			error_helpers.ShowErrorWithMessage(ctx, err, "failed to reload mod from file watcher")
 		}
 	}
 
@@ -191,6 +201,43 @@ func (w *Workspace) ModfileExists() bool {
 	return len(w.modFilePath) > 0
 }
 
+func (w *Workspace) HandlePreparedStatementFailures(failures *db_common.PrepareStatementFailures) {
+	if failures == nil {
+		return
+	}
+
+	// avoid concurrent map access when multiple db connections may try to access preparedStatementFailures
+	w.preparedStatementFailureLock.Lock()
+	defer w.preparedStatementFailureLock.Unlock()
+
+	// replace the map of failures with the current map
+	w.preparedStatementFailures = make(map[string]*steampipeconfig.PreparedStatementFailure)
+	for queryName, err := range failures.Failures {
+		if query, ok := w.GetQueryProvider(queryName); ok {
+			w.preparedStatementFailures[queryName] = &steampipeconfig.PreparedStatementFailure{
+				Query: query,
+				Error: err,
+			}
+		}
+	}
+	if failures.Error != nil {
+		w.preparedStatementFailures["preparedStatementGlobalError"] = &steampipeconfig.PreparedStatementFailure{
+			Error: failures.Error,
+		}
+
+	}
+}
+
+// GetPreparedStatementCreationFailure looks for a prepared statement error for the given query and if found,
+// returns the query and the prepared statement creation error (if any)
+func (w *Workspace) GetPreparedStatementCreationFailure(queryName string) *steampipeconfig.PreparedStatementFailure {
+	return w.preparedStatementFailures[queryName]
+}
+
+func (w *Workspace) GetPreparedStatementFailures() map[string]*steampipeconfig.PreparedStatementFailure {
+	return w.preparedStatementFailures
+}
+
 // check  whether the workspace contains a modfile
 // this will determine whether we load files recursively, and create pseudo resources for sql files
 func (w *Workspace) setModfileExists() {
@@ -205,7 +252,7 @@ func (w *Workspace) setModfileExists() {
 		w.modFilePath = modFile
 
 		// also set it in the viper config, so that it is available to whoever is using it
-		viper.Set(constants.ArgWorkspaceChDir, filepath.Dir(modFile))
+		viper.Set(constants.ArgModLocation, filepath.Dir(modFile))
 		w.Path = filepath.Dir(modFile)
 	} else {
 		log.Printf("[TRACE] no modfile exists in workspace folder - NOT creating pseudoresources and only loading resource files from top level folder")
@@ -250,17 +297,17 @@ func (w *Workspace) loadWorkspaceMod(ctx context.Context) error {
 	w.VariableValues = inputVariables.VariableValues
 
 	// build run context which we use to load the workspace
-	runCtx, err := w.getRunContext()
+	parseCtx, err := w.getParseContext()
 	if err != nil {
 		return err
 	}
 	// add variables to runContext
-	runCtx.AddInputVariables(inputVariables)
+	parseCtx.AddInputVariables(inputVariables)
 	// do not reload variables as we already have them
-	runCtx.BlockTypeExclusions = []string{modconfig.BlockTypeVariable}
+	parseCtx.BlockTypeExclusions = []string{modconfig.BlockTypeVariable}
 
 	// load the workspace mod
-	m, err := steampipeconfig.LoadMod(w.Path, runCtx)
+	m, err := steampipeconfig.LoadMod(w.Path, parseCtx)
 	if err != nil {
 		return err
 	}
@@ -270,7 +317,7 @@ func (w *Workspace) loadWorkspaceMod(ctx context.Context) error {
 	m.ResourceMaps.PopulateReferences()
 	// set the mod
 	w.Mod = m
-	w.Mods = runCtx.LoadedDependencyMods
+	w.Mods = parseCtx.LoadedDependencyMods
 	// NOTE: add in the workspace mod to the dependency mods
 	w.Mods[w.Mod.Name()] = w.Mod
 
@@ -280,7 +327,7 @@ func (w *Workspace) loadWorkspaceMod(ctx context.Context) error {
 
 func (w *Workspace) getInputVariables(ctx context.Context, validateMissing bool) (*modconfig.ModVariableMap, error) {
 	// build a run context just to use to load variable definitions
-	variablesRunCtx, err := w.getRunContext()
+	variablesRunCtx, err := w.getParseContext()
 	if err != nil {
 		return nil, err
 	}
@@ -296,7 +343,7 @@ func (w *Workspace) getInputVariables(ctx context.Context, validateMissing bool)
 
 // build options used to load workspace
 // set flags to create pseudo resources and a default mod if needed
-func (w *Workspace) getRunContext() (*parse.RunContext, error) {
+func (w *Workspace) getParseContext() (*parse.ModParseContext, error) {
 	parseFlag := parse.CreateDefaultMod
 	if w.loadPseudoResources {
 		parseFlag |= parse.CreatePseudoResources
@@ -305,7 +352,7 @@ func (w *Workspace) getRunContext() (*parse.RunContext, error) {
 	if err != nil {
 		return nil, err
 	}
-	runCtx := parse.NewRunContext(
+	parseCtx := parse.NewModParseContext(
 		workspaceLock,
 		w.Path,
 		parseFlag,
@@ -317,7 +364,7 @@ func (w *Workspace) getRunContext() (*parse.RunContext, error) {
 			Include: filehelpers.InclusionsFromExtensions([]string{constants.ModDataExtension}),
 		})
 
-	return runCtx, nil
+	return parseCtx, nil
 }
 
 // load the workspace lock, migrating it if necessary
@@ -329,7 +376,7 @@ func (w *Workspace) loadWorkspaceLock() (*versionmap.WorkspaceLock, error) {
 
 	// if this is the old format, migrate by reinstalling dependencies
 	if workspaceLock.StructVersion() != versionmap.WorkspaceLockStructVersion {
-		opts := &modinstaller.InstallOpts{WorkspacePath: viper.GetString(constants.ArgWorkspaceChDir)}
+		opts := &modinstaller.InstallOpts{WorkspacePath: viper.GetString(constants.ArgModLocation)}
 		installData, err := modinstaller.InstallWorkspaceDependencies(opts)
 		if err != nil {
 			return nil, err
@@ -376,23 +423,16 @@ func (w *Workspace) loadExclusions() error {
 
 func (w *Workspace) loadWorkspaceResourceName() (*modconfig.WorkspaceResources, error) {
 	// build options used to load workspace
-	runCtx, err := w.getRunContext()
+	parseCtx, err := w.getParseContext()
 	if err != nil {
 		return nil, err
 	}
 
-	workspaceResourceNames, err := steampipeconfig.LoadModResourceNames(w.Path, runCtx)
+	workspaceResourceNames, err := steampipeconfig.LoadModResourceNames(w.Path, parseCtx)
 	if err != nil {
 		return nil, err
 	}
 
-	// TODO load resource names for dependency mods
-	//modsPath := file_paths.WorkspaceModPath(w.Path)
-	//dependencyResourceNames, err := w.loadModDependencyResourceNames(modsPath)
-	//if err != nil {
-	//	return nil, err
-	//}
-
 	return workspaceResourceNames, nil
 }
 
