@@ -1,28 +1,46 @@
 import get from "lodash/get";
-import has from "lodash/has";
 import isEqual from "lodash/isEqual";
 import paths from "deepdash/paths";
-import set from "lodash/set";
-import sortBy from "lodash/sortBy";
-import useDashboardWebSocket, {
-  SocketActions,
-  SocketURLFactory,
-} from "./useDashboardWebSocket";
-import usePrevious from "./usePrevious";
-import { buildComponentsMap } from "../components";
-import {
+import React, {
   createContext,
-  Ref,
   useCallback,
   useContext,
   useEffect,
   useReducer,
   useState,
 } from "react";
+import set from "lodash/set";
+import sortBy from "lodash/sortBy";
+import useDashboardWebSocket, { SocketActions } from "./useDashboardWebSocket";
+import useDashboardWebSocketEventHandler from "./useDashboardWebSocketEventHandler";
+import usePrevious from "./usePrevious";
+import {
+  AvailableDashboard,
+  AvailableDashboardsDictionary,
+  DashboardActions,
+  DashboardDataModeCLISnapshot,
+  DashboardDataModeCloudSnapshot,
+  DashboardDataModeLive,
+  DashboardDataOptions,
+  DashboardDefinition,
+  DashboardRenderOptions,
+  DashboardsCollection,
+  IDashboardContext,
+  PanelDefinition,
+  PanelsMap,
+  SelectedDashboardStates,
+  SocketURLFactory,
+  SQLDataMap,
+} from "../types";
+import { buildComponentsMap } from "../components";
+import {
+  controlsUpdatedEventHandler,
+  leafNodesCompleteEventHandler,
+  migrateDashboardExecutionCompleteSchema,
+} from "../utils/dashboardEventHandlers";
 import { GlobalHotKeys } from "react-hotkeys";
-import { LeafNodeData, Width } from "../components/dashboards/common";
+import { KeyValueStringPairs } from "../components/dashboards/common/types";
 import { noop } from "../utils/func";
-import { Theme } from "./useTheme";
 import {
   useLocation,
   useNavigate,
@@ -30,302 +48,11 @@ import {
   useParams,
   useSearchParams,
 } from "react-router-dom";
-import isEmpty from "lodash/isEmpty";
-
-interface IBreakpointContext {
-  currentBreakpoint: string | null;
-  maxBreakpoint(breakpointAndDown: string): boolean;
-  minBreakpoint(breakpointAndUp: string): boolean;
-  width: number;
-}
-
-interface IThemeContext {
-  theme: Theme;
-  setTheme(theme: string): void;
-  wrapperRef: Ref<null>;
-}
-
-export interface ComponentsMap {
-  [name: string]: any;
-}
-
-export interface PanelsMap {
-  [name: string]: PanelDefinition;
-}
-
-export type DashboardDataMode = "live" | "snapshot";
-
-export type DashboardRunState = "ready" | "error" | "complete";
-
-interface IDashboardContext {
-  metadata: DashboardMetadata | null;
-  availableDashboardsLoaded: boolean;
-
-  closePanelDetail(): void;
-  dispatch(action: DashboardAction): void;
-
-  dataMode: DashboardDataMode;
-  snapshotId: string | null;
-
-  refetchDashboard: boolean;
-
-  error: any;
-
-  panelsMap: PanelsMap;
-
-  dashboards: AvailableDashboard[];
-  dashboardsMap: AvailableDashboardsDictionary;
-  dashboard: DashboardDefinition | null;
-
-  selectedPanel: PanelDefinition | null;
-  selectedDashboard: AvailableDashboard | null;
-  selectedDashboardInputs: DashboardInputs;
-  selectedSnapshot: DashboardSnapshot | null;
-  lastChangedInput: string | null;
-
-  sqlDataMap: SQLDataMap;
-
-  dashboardTags: DashboardTags;
-
-  search: DashboardSearch;
-
-  breakpointContext: IBreakpointContext;
-  themeContext: IThemeContext;
-
-  components: ComponentsMap;
-
-  progress: number;
-  state: DashboardRunState;
-}
-
-export interface IActions {
-  [type: string]: string;
-}
-
-const DashboardActions: IActions = {
-  AVAILABLE_DASHBOARDS: "available_dashboards",
-  CLEAR_DASHBOARD_INPUTS: "clear_dashboard_inputs",
-  CLEAR_SNAPSHOT: "clear_snapshot",
-  CONTROL_COMPLETE: "control_complete",
-  CONTROL_ERROR: "control_error",
-  DASHBOARD_METADATA: "dashboard_metadata",
-  DELETE_DASHBOARD_INPUT: "delete_dashboard_input",
-  EXECUTION_COMPLETE: "execution_complete",
-  EXECUTION_ERROR: "execution_error",
-  EXECUTION_STARTED: "execution_started",
-  INPUT_VALUES_CLEARED: "input_values_cleared",
-  LEAF_NODE_COMPLETE: "leaf_node_complete",
-  LEAF_NODE_PROGRESS: "leaf_node_progress",
-  SELECT_DASHBOARD: "select_dashboard",
-  SELECT_PANEL: "select_panel",
-  SELECT_SNAPSHOT: "select_snapshot",
-  SET_DASHBOARD: "set_dashboard",
-  SET_DASHBOARD_INPUT: "set_dashboard_input",
-  SET_DASHBOARD_INPUTS: "set_dashboard_inputs",
-  SET_DASHBOARD_SEARCH_VALUE: "set_dashboard_search_value",
-  SET_DASHBOARD_SEARCH_GROUP_BY: "set_dashboard_search_group_by",
-  SET_DASHBOARD_TAG_KEYS: "set_dashboard_tag_keys",
-  SET_DATA_MODE: "set_data_mode",
-  SET_REFETCH_DASHBOARD: "set_refetch_dashboard",
-  SET_SNAPSHOT: "set_snapshot",
-  WORKSPACE_ERROR: "workspace_error",
-};
-
-const dashboardActions = Object.values(DashboardActions);
-
-// https://github.com/microsoft/TypeScript/issues/28046
-export type ElementType<T extends ReadonlyArray<unknown>> =
-  T extends ReadonlyArray<infer ElementType> ? ElementType : never;
-
-export type DashboardActionType = ElementType<typeof dashboardActions>;
-
-export interface DashboardAction {
-  type: DashboardActionType;
-  [key: string]: any;
-}
-
-type DashboardSearchGroupByMode = "mod" | "tag";
-
-interface DashboardSearchGroupBy {
-  value: DashboardSearchGroupByMode;
-  tag: string | null;
-}
-
-export interface DashboardSearch {
-  value: string;
-  groupBy: DashboardSearchGroupBy;
-}
-
-export interface DashboardTags {
-  keys: string[];
-}
-
-interface SelectedDashboardStates {
-  dashboard_name: string | null;
-  dataMode: DashboardDataMode;
-  refetchDashboard: boolean;
-  search: DashboardSearch;
-  selectedDashboard: AvailableDashboard | null;
-  selectedDashboardInputs: DashboardInputs;
-  selectedSnapshot: DashboardSnapshot;
-}
-
-interface DashboardInputs {
-  [name: string]: string;
-}
-
-interface DashboardVariables {
-  [name: string]: any;
-}
-
-export interface ModDashboardMetadata {
-  title: string;
-  full_name: string;
-  short_name: string;
-}
-
-interface InstalledModsDashboardMetadata {
-  [key: string]: ModDashboardMetadata;
-}
-
-export interface CloudDashboardActorMetadata {
-  id: string;
-  handle: string;
-}
-
-export interface CloudDashboardIdentityMetadata {
-  id: string;
-  handle: string;
-  type: "org" | "user";
-}
-
-export interface CloudDashboardWorkspaceMetadata {
-  id: string;
-  handle: string;
-}
-
-interface CloudDashboardMetadata {
-  actor: CloudDashboardActorMetadata;
-  identity: CloudDashboardIdentityMetadata;
-  workspace: CloudDashboardWorkspaceMetadata;
-}
-
-export interface DashboardMetadata {
-  mod: ModDashboardMetadata;
-  installed_mods?: InstalledModsDashboardMetadata;
-  cloud?: CloudDashboardMetadata;
-  telemetry: "info" | "none";
-}
-
-export interface DashboardSnapshot {
-  id: string;
-  dashboard_name: string;
-  start_time: string;
-  end_time: string;
-  schema_version: string;
-  search_path: string;
-  variables: DashboardVariables;
-  inputs: DashboardInputs;
-}
-
-interface AvailableDashboardTags {
-  [key: string]: string;
-}
-
-type AvailableDashboardType = "benchmark" | "dashboard";
-
-export interface AvailableDashboard {
-  full_name: string;
-  short_name: string;
-  mod_full_name: string;
-  tags: AvailableDashboardTags;
-  title: string;
-  is_top_level: boolean;
-  type: AvailableDashboardType;
-  children?: AvailableDashboard[];
-  trunks?: string[][];
-}
-
-export interface AvailableDashboardsDictionary {
-  [key: string]: AvailableDashboard;
-}
-
-export interface ContainerDefinition {
-  name: string;
-  panel_type?: string;
-  allow_child_panel_expand?: boolean;
-  data?: LeafNodeData;
-  title?: string;
-  width?: number;
-  children?: (ContainerDefinition | PanelDefinition)[];
-}
-
-export interface PanelProperties {
-  [key: string]: any;
-}
-
-export interface SQLDataMap {
-  [sql: string]: LeafNodeData;
-}
-
-export interface PanelDefinition {
-  name: string;
-  display_type?: string;
-  panel_type?: string;
-  title?: string;
-  description?: string;
-  width?: Width;
-  sql?: string;
-  data?: LeafNodeData;
-  source_definition?: string;
-  status?: DashboardRunState;
-  error?: Error;
-  properties?: PanelProperties;
-  dashboard: string;
-}
-
-export interface BenchmarkDefinition extends PanelDefinition {
-  children?: BenchmarkDefinition | ControlDefinition[];
-}
-
-export interface ControlDefinition extends PanelDefinition {}
-
-export interface DashboardDefinition {
-  artificial: boolean;
-  name: string;
-  panel_type: string;
-  title?: string;
-  width?: number;
-  children?: (ContainerDefinition | PanelDefinition)[];
-  dashboard: string;
-}
-
-interface DashboardsCollection {
-  dashboards: AvailableDashboard[];
-  dashboardsMap: AvailableDashboardsDictionary;
-}
-
-export interface DashboardDataOptions {
-  dataMode: DashboardDataMode;
-  snapshotId?: string;
-}
-
-interface DashboardProviderProps {
-  analyticsContext: any;
-  breakpointContext: any;
-  children: null | JSX.Element | JSX.Element[];
-  componentOverrides?: {};
-  dataOptions?: DashboardDataOptions;
-  eventHooks?: {};
-  featureFlags?: string[];
-  socketUrlFactory?: SocketURLFactory;
-  stateDefaults?: {};
-  themeContext: any;
-}
 
 const buildDashboards = (
   dashboards: AvailableDashboardsDictionary,
-  benchmarks: AvailableDashboardsDictionary
+  benchmarks: AvailableDashboardsDictionary,
+  snapshots: KeyValueStringPairs
 ): DashboardsCollection => {
   const dashboardsMap = {};
   const builtDashboards: AvailableDashboard[] = [];
@@ -360,6 +87,19 @@ const buildDashboards = (
     builtDashboards.push(builtBenchmark);
   }
 
+  for (const snapshot of Object.keys(snapshots || {})) {
+    const builtSnapshot: AvailableDashboard = {
+      title: snapshot,
+      full_name: snapshot,
+      short_name: snapshot,
+      type: "snapshot",
+      tags: {},
+      is_top_level: true,
+    };
+    dashboardsMap[builtSnapshot.full_name] = builtSnapshot;
+    builtDashboards.push(builtSnapshot);
+  }
+
   return {
     dashboards: sortBy(builtDashboards, [
       (dashboard) =>
@@ -425,11 +165,11 @@ function addDataToPanels(panels: PanelsMap, sqlDataMap: SQLDataMap): PanelsMap {
       set(panels, dataPath, data);
     }
   }
-  return panels;
+  return { ...panels };
 }
 
 const wrapDefinitionInArtificialDashboard = (
-  definition: DashboardDefinition,
+  definition: PanelDefinition,
   layout: any
 ): DashboardDefinition => {
   const { title: defTitle, ...definitionWithoutTitle } = definition;
@@ -449,48 +189,21 @@ const wrapDefinitionInArtificialDashboard = (
   };
 };
 
-const updatePanelsMapWithControlEvent = (panelsMap, action) => {
-  return {
-    ...panelsMap,
-    [action.control.name]: action.control,
-  };
-};
-
-const calculateProgress = (panelsMap) => {
-  const panels: PanelDefinition[] = Object.values(panelsMap || {});
-  let dataPanels = 0;
-  let completeDataPanels = 0;
-  for (const panel of panels) {
-    const isControl = panel.panel_type === "control";
-    const isDataPanel = has(panel, "sql");
-    if (isControl || isDataPanel) {
-      dataPanels += 1;
-    }
-    if (
-      (isControl &&
-        (panel.status === "complete" || panel.status === "error")) ||
-      (isDataPanel && has(panel, "data"))
-    ) {
-      completeDataPanels += 1;
-    }
-  }
-  if (dataPanels === 0) {
-    return 100;
-  }
-  return Math.min(Math.ceil((completeDataPanels / dataPanels) * 100), 100);
-};
-
 function reducer(state, action) {
   switch (action.type) {
     case DashboardActions.DASHBOARD_METADATA:
       return {
         ...state,
-        metadata: action.metadata,
+        metadata: {
+          mod: {},
+          ...action.metadata,
+        },
       };
     case DashboardActions.AVAILABLE_DASHBOARDS:
       const { dashboards, dashboardsMap } = buildDashboards(
         action.dashboards,
-        action.benchmarks
+        action.benchmarks,
+        action.snapshots
       );
       const selectedDashboard = updateSelectedDashboard(
         state.selectedDashboard,
@@ -503,11 +216,13 @@ function reducer(state, action) {
         dashboards,
         dashboardsMap,
         selectedDashboard:
-          state.dataMode === "snapshot"
+          state.dataMode === DashboardDataModeCLISnapshot ||
+          state.dataMode === DashboardDataModeCloudSnapshot
             ? state.selectedDashboard
             : selectedDashboard,
         dashboard:
-          state.dataMode === "snapshot"
+          state.dataMode === DashboardDataModeCLISnapshot ||
+          state.dataMode === DashboardDataModeCloudSnapshot
             ? state.dashboard
             : selectedDashboard &&
               state.dashboard &&
@@ -516,19 +231,20 @@ function reducer(state, action) {
             : null,
       };
     case DashboardActions.EXECUTION_STARTED: {
-      const originalDashboard = action.dashboard_node;
+      const rootLayoutPanel = action.layout;
+      const rootPanel = action.panels[rootLayoutPanel.name];
       let dashboard;
       // For benchmarks and controls that are run directly from a mod,
       // we need to wrap these in an artificial dashboard, so we can treat
       // it just like any other dashboard
-      if (action.dashboard_node.panel_type !== "dashboard") {
+      if (rootPanel.panel_type !== "dashboard") {
         dashboard = wrapDefinitionInArtificialDashboard(
-          originalDashboard,
+          rootPanel,
           action.layout
         );
       } else {
         dashboard = {
-          ...originalDashboard,
+          ...rootPanel,
           ...action.layout,
         };
       }
@@ -541,109 +257,73 @@ function reducer(state, action) {
         execution_id: action.execution_id,
         refetchDashboard: false,
         progress: 0,
+        snapshot: null,
         state: "ready",
       };
     }
     case DashboardActions.EXECUTION_COMPLETE: {
       // If we're in live mode and not expecting execution events for this ID
       if (
-        state.dataMode === "live" &&
+        state.dataMode === DashboardDataModeLive &&
         action.execution_id !== state.execution_id
       ) {
         return state;
       }
 
-      const originalDashboard = action.dashboard_node;
+      const migratedEvent = migrateDashboardExecutionCompleteSchema(action);
+      const layout = migratedEvent.snapshot.layout;
+      const panels = migratedEvent.snapshot.panels;
+      const rootLayoutPanel = migratedEvent.snapshot.layout;
+      const rootPanel = panels[rootLayoutPanel.name];
       let dashboard;
 
-      if (action.dashboard_node.panel_type !== "dashboard") {
-        dashboard = wrapDefinitionInArtificialDashboard(
-          originalDashboard,
-          action.layout
-        );
+      if (rootPanel.panel_type !== "dashboard") {
+        dashboard = wrapDefinitionInArtificialDashboard(rootPanel, layout);
       } else {
         dashboard = {
-          ...originalDashboard,
-          ...action.layout,
+          ...rootPanel,
+          ...layout,
         };
       }
 
       // Build map of SQL to data
-      const sqlDataMap = buildSqlDataMap(action.panels);
+      const sqlDataMap = buildSqlDataMap(panels);
       // Replace the whole dashboard as this event contains everything
       return {
         ...state,
         error: null,
-        panelsMap: action.panels,
+        panelsMap: panels,
         dashboard,
         sqlDataMap,
         progress: 100,
+        snapshot: action.snapshot,
         state: "complete",
       };
     }
     case DashboardActions.EXECUTION_ERROR:
       return { ...state, error: action.error, progress: 100, state: "error" };
-    case DashboardActions.CONTROL_COMPLETE:
-    case DashboardActions.CONTROL_ERROR:
-      // We're not expecting execution events for this ID
-      if (action.execution_id !== state.execution_id) {
-        return state;
-      }
-
-      const updatedPanelsMap = updatePanelsMapWithControlEvent(
-        state.panelsMap,
-        action
-      );
-
-      if (!updatedPanelsMap) {
-        return state;
-      }
-
-      return {
-        ...state,
-        panelsMap: updatedPanelsMap,
-        progress: calculateProgress(updatedPanelsMap),
-      };
-    case DashboardActions.LEAF_NODE_COMPLETE: {
-      // We're not expecting execution events for this ID
-      if (action.execution_id !== state.execution_id) {
-        return state;
-      }
-
-      const { dashboard_node } = action;
-
-      const panelsMap = {
-        ...state.panelsMap,
-        [dashboard_node.name]: dashboard_node,
-      };
-
-      return {
-        ...state,
-        panelsMap,
-        progress: calculateProgress(panelsMap),
-      };
-    }
+    case DashboardActions.CONTROLS_UPDATED:
+      return controlsUpdatedEventHandler(action, state);
+    case DashboardActions.LEAF_NODES_COMPLETE:
+      return leafNodesCompleteEventHandler(action, state);
     case DashboardActions.SELECT_PANEL:
       return { ...state, selectedPanel: action.panel };
-    case DashboardActions.CLEAR_SNAPSHOT:
-      return {
-        ...state,
-        selectedSnapshot: null,
-        snapshotId: null,
-        dataMode: "live",
-      };
-    case DashboardActions.SELECT_SNAPSHOT:
-      return {
-        ...state,
-        selectedSnapshot: action.snapshot,
-        snapshotId: action.snapshot.id,
-        dataMode: "snapshot",
-      };
     case DashboardActions.SET_DATA_MODE:
-      return {
+      const newState = {
         ...state,
         dataMode: action.dataMode,
       };
+      if (action.dataMode === DashboardDataModeCLISnapshot) {
+        newState.snapshotFileName = action.snapshotFileName;
+      } else if (
+        state.dataMode !== DashboardDataModeLive &&
+        action.dataMode === DashboardDataModeLive
+      ) {
+        newState.snapshot = null;
+        newState.snapshotFileName = null;
+        newState.snapshotId = null;
+      }
+      return newState;
     case DashboardActions.SET_REFETCH_DASHBOARD:
       return {
         ...state,
@@ -655,19 +335,35 @@ function reducer(state, action) {
         dashboard: action.dashboard,
       };
     case DashboardActions.SELECT_DASHBOARD:
-      if (action.dataMode === "snapshot") {
+      if (action.dashboard && action.dashboard.type === "snapshot") {
         return {
           ...state,
-          dataMode: "snapshot",
+          dataMode: DashboardDataModeCLISnapshot,
           selectedDashboard: action.dashboard,
         };
       }
+
+      if (
+        action.dataMode === DashboardDataModeCLISnapshot ||
+        action.dataMode === DashboardDataModeCloudSnapshot
+      ) {
+        return {
+          ...state,
+          dataMode: action.dataMode,
+          selectedDashboard: action.dashboard,
+        };
+      }
+
       return {
         ...state,
-        dataMode: "live",
+        dataMode: DashboardDataModeLive,
         dashboard: null,
         execution_id: null,
+        panelsMap: {},
+        snapshot: null,
+        snapshotFileName: null,
         snapshotId: null,
+        sqlDataMap: {},
         state: null,
         selectedDashboard: action.dashboard,
         selectedPanel: null,
@@ -793,7 +489,7 @@ const getInitialState = (searchParams, defaults: any = {}) => {
     dashboardTags: {
       keys: [],
     },
-    dataMode: defaults.dataMode || "live",
+    dataMode: defaults.dataMode || DashboardDataModeLive,
     snapshotId: defaults.snapshotId ? defaults.snapshotId : null,
     refetchDashboard: false,
     error: null,
@@ -803,7 +499,7 @@ const getInitialState = (searchParams, defaults: any = {}) => {
     selectedDashboard: null,
     selectedDashboardInputs:
       buildSelectedDashboardInputsFromSearchParams(searchParams),
-    selectedSnapshot: null,
+    snapshot: null,
     lastChangedInput: null,
 
     search: {
@@ -828,16 +524,33 @@ const getInitialState = (searchParams, defaults: any = {}) => {
 
 const DashboardContext = createContext<IDashboardContext | null>(null);
 
+interface DashboardProviderProps {
+  analyticsContext: any;
+  breakpointContext: any;
+  children: null | JSX.Element | JSX.Element[];
+  componentOverrides?: {};
+  dataOptions?: DashboardDataOptions;
+  eventHooks?: {};
+  featureFlags?: string[];
+  renderOptions?: DashboardRenderOptions;
+  socketUrlFactory?: SocketURLFactory;
+  stateDefaults?: {};
+  themeContext: any;
+}
+
 const DashboardProvider = ({
   analyticsContext,
   breakpointContext,
   children,
   componentOverrides = {},
   dataOptions = {
-    dataMode: "live",
+    dataMode: DashboardDataModeLive,
   },
   eventHooks,
   featureFlags = [],
+  renderOptions = {
+    headless: false,
+  },
   socketUrlFactory,
   stateDefaults = {},
   themeContext,
@@ -847,17 +560,25 @@ const DashboardProvider = ({
   const [searchParams, setSearchParams] = useSearchParams();
   const [state, dispatchInner] = useReducer(
     reducer,
-    getInitialState(searchParams, { ...stateDefaults, ...dataOptions })
+    getInitialState(searchParams, {
+      ...stateDefaults,
+      ...dataOptions,
+      ...renderOptions,
+    })
   );
   const dispatch = useCallback((action) => {
     // console.log(action.type, action);
     dispatchInner(action);
   }, []);
   const { dashboard_name } = useParams();
+  const { eventHandler } = useDashboardWebSocketEventHandler(
+    dispatch,
+    eventHooks
+  );
   const { ready: socketReady, send: sendSocketMessage } = useDashboardWebSocket(
     state.dataMode,
     dispatch,
-    eventHooks,
+    eventHandler,
     socketUrlFactory
   );
   const {
@@ -878,7 +599,6 @@ const DashboardProvider = ({
       search: state.search,
       selectedDashboard: state.selectedDashboard,
       selectedDashboardInputs: state.selectedDashboardInputs,
-      selectedSnapshot: state.selectedSnapshot,
     });
 
   // Alert analytics
@@ -890,6 +610,19 @@ const DashboardProvider = ({
     setAnalyticsSelectedDashboard(state.selectedDashboard);
   }, [state.selectedDashboard, setAnalyticsSelectedDashboard]);
 
+  useEffect(() => {
+    if (
+      !!dashboard_name &&
+      !location.pathname.startsWith("/snapshot/") &&
+      state.dataMode === DashboardDataModeCLISnapshot
+    ) {
+      dispatch({
+        type: DashboardActions.SET_DATA_MODE,
+        dataMode: DashboardDataModeLive,
+      });
+    }
+  }, [dashboard_name, dispatch, location, navigate, state.dataMode]);
+
   // Ensure that on history pop / push we sync the new values into state
   useEffect(() => {
     if (navigationType !== "POP" && navigationType !== "PUSH") {
@@ -898,7 +631,7 @@ const DashboardProvider = ({
     if (location.key === "default") {
       return;
     }
-    if (state.dataMode === "snapshot") {
+    if (state.dataMode !== DashboardDataModeLive) {
       return;
     }
 
@@ -919,11 +652,6 @@ const DashboardProvider = ({
       searchParams.get("tag") ||
       get(stateDefaults, "search.groupBy.tag", "service");
     const inputs = buildSelectedDashboardInputsFromSearchParams(searchParams);
-    // console.log({
-    //   // @ts-ignore
-    //   previous: previousSelectedDashboardStates?.selectedDashboardInputs,
-    //   current: inputs,
-    // });
     dispatch({
       type: DashboardActions.SET_DASHBOARD_SEARCH_VALUE,
       value: goneFromDashboardToDashboard ? "" : search,
@@ -939,7 +667,6 @@ const DashboardProvider = ({
         previousSelectedDashboardStates?.selectedDashboardInputs
       ) !== JSON.stringify(inputs)
     ) {
-      // console.log("dispatching inputs", inputs);
       dispatch({
         type: DashboardActions.SET_DASHBOARD_INPUTS,
         value: inputs,
@@ -961,7 +688,8 @@ const DashboardProvider = ({
   useEffect(() => {
     // If no search params have changed
     if (
-      state.dataMode === "snapshot" ||
+      state.dataMode === DashboardDataModeCloudSnapshot ||
+      state.dataMode === DashboardDataModeCLISnapshot ||
       (previousSelectedDashboardStates &&
         // @ts-ignore
         previousSelectedDashboardStates?.dashboard_name === dashboard_name &&
@@ -1053,14 +781,17 @@ const DashboardProvider = ({
     }
     // Else if we've got a dashboard selected in the URL and don't have one selected in state,
     // select that dashboard
-    if (dashboard_name && !state.selectedDashboard) {
+    if (
+      dashboard_name &&
+      !state.selectedDashboard &&
+      state.dataMode === DashboardDataModeLive
+    ) {
       const dashboard = state.dashboards.find(
         (dashboard) => dashboard.full_name === dashboard_name
       );
       dispatch({
         type: DashboardActions.SELECT_DASHBOARD,
         dashboard,
-        dataMode: state.dataMode,
       });
       return;
     }
@@ -1091,6 +822,20 @@ const DashboardProvider = ({
     state.selectedDashboard,
   ]);
 
+  useEffect(() => {
+    if (
+      !dashboard_name &&
+      state.snapshot &&
+      state.dataMode === DashboardDataModeCLISnapshot
+    ) {
+      dispatch({
+        type: DashboardActions.SELECT_DASHBOARD,
+        dashboard: null,
+        dataMode: DashboardDataModeLive,
+      });
+    }
+  }, [dashboard_name, dispatch, state.dataMode, state.snapshot]);
+
   useEffect(() => {
     // This effect will send events over websockets and depends on there being a dashboard selected
     if (!socketReady || !state.selectedDashboard) {
@@ -1101,7 +846,8 @@ const DashboardProvider = ({
     // to a report, or it's first load), or the selected dashboard has been changed, select that
     // report over the socket
     if (
-      state.dataMode === "live" &&
+      (state.dataMode === DashboardDataModeLive ||
+        state.dataMode === DashboardDataModeCLISnapshot) &&
       (!previousSelectedDashboardStates ||
         // @ts-ignore
         !previousSelectedDashboardStates.selectedDashboard ||
@@ -1116,7 +862,10 @@ const DashboardProvider = ({
         action: SocketActions.CLEAR_DASHBOARD,
       });
       sendSocketMessage({
-        action: SocketActions.SELECT_DASHBOARD,
+        action:
+          state.selectedDashboard.type === "snapshot"
+            ? SocketActions.SELECT_SNAPSHOT
+            : SocketActions.SELECT_DASHBOARD,
         payload: {
           dashboard: {
             full_name: state.selectedDashboard.full_name,
@@ -1129,7 +878,7 @@ const DashboardProvider = ({
     // Else if we did previously have a dashboard selected in state and the
     // inputs have changed, then update the inputs over the socket
     if (
-      state.dataMode === "live" &&
+      state.dataMode === DashboardDataModeLive &&
       previousSelectedDashboardStates &&
       // @ts-ignore
       previousSelectedDashboardStates.selectedDashboard &&
@@ -1233,9 +982,14 @@ const DashboardProvider = ({
   ]);
 
   useEffect(() => {
-    if (!state.availableDashboardsLoaded || !dashboard_name) {
+    if (
+      !state.availableDashboardsLoaded ||
+      !dashboard_name ||
+      state.dataMode === DashboardDataModeCLISnapshot
+    ) {
       return;
     }
+
     // If the dashboard we're viewing no longer exists, go back to the main page
     if (!state.dashboards.find((r) => r.full_name === dashboard_name)) {
       navigate("../", { replace: true });
@@ -1245,8 +999,18 @@ const DashboardProvider = ({
     dashboard_name,
     state.availableDashboardsLoaded,
     state.dashboards,
+    state.dataMode,
   ]);
 
+  useEffect(() => {
+    if (
+      location.pathname.startsWith("/snapshot/") &&
+      state.dataMode !== DashboardDataModeCLISnapshot
+    ) {
+      navigate("/");
+    }
+  }, [location, navigate, state.dataMode]);
+
   useEffect(() => {
     if (!state.selectedDashboard) {
       document.title = "Dashboards | Steampipe";
@@ -1278,6 +1042,20 @@ const DashboardProvider = ({
     });
   }, [closePanelDetail]);
 
+  const [renderSnapshotCompleteDiv, setRenderSnapshotCompleteDiv] =
+    useState(false);
+
+  useEffect(() => {
+    if (
+      (dataOptions?.dataMode !== DashboardDataModeCLISnapshot &&
+        dataOptions?.dataMode !== DashboardDataModeCloudSnapshot) ||
+      state.state !== "complete"
+    ) {
+      return;
+    }
+    setRenderSnapshotCompleteDiv(true);
+  }, [dataOptions?.dataMode, state.state]);
+
   return (
     <DashboardContext.Provider
       value={{
@@ -1288,6 +1066,10 @@ const DashboardProvider = ({
         dispatch,
         closePanelDetail,
         themeContext,
+        render: {
+          headless: renderOptions?.headless,
+          snapshotCompleteDiv: renderSnapshotCompleteDiv,
+        },
       }}
     >
       <GlobalHotKeys
@@ -1308,4 +1090,4 @@ const useDashboard = () => {
   return context as IDashboardContext;
 };
 
-export { DashboardActions, DashboardContext, DashboardProvider, useDashboard };
+export { DashboardContext, DashboardProvider, useDashboard };
